//version 0.3 Julian Rohrhuber 2001/05 
//a streamclass derived from Langton's Ants, similar to Turing Machines
//a related concept is the TURTLE (LOGO) see also Turtle.sc
//the array2D should contain ControlValues that can be set
//ControlArray2D provides easy creation of this.

Ant : Turtle {
	var <>array2D; 
	var <>function;
	
	*new { arg array2D, mode=\default, stream, event;
			^super.new(stream, event ?? AntEvent.protoEvent)
			.array2D_(array2D).init(mode)
	}
	init { arg mode; 
			stream = stream ?? { Pbind.new.asStream };
			this.perform(mode) 
	}
					
	
	next { arg inval; ^function.value(inval) }
	step { 	
			event.use({
				
			var x, y;
			x = (~direction*pi).sin * ~step + ~x;
			y = (~direction*pi).cos * ~step + ~y;
			~x = x.round(1).wrap(0, array2D.cols).asInteger;
			~y = y.round(1).wrap(0, array2D.rows).asInteger;
			
			})
	}
	set { 	//this method works for an array2D of ControlValues.
			event.use({ array2D.put(~y.asInteger, ~x.asInteger, ~value)  })
	}
	
	turn { event.use({
				~direction = (~direction + ~turn).wrap(0.0, 2.0);
				})
	}
	
	position { 
			^Point(event.at(\x), event.at(\y))
	}
	
	
	
	
	//-----these are the modes accessed by init with mode argument:
	
	
	//Langtons Ant with two states
	langton { 
			function = {
				event = stream.next(event);
				
				
				event.use({
				
				if(array2D.at(~y, ~x) > ~thresh, 
				
					{ 	 
						~turn = ~turn2; 
						~value = ~value2;
						
					},
					{
						~turn = ~turn1;
						~value = ~value1;
						
					}
				);
				
				
				
				});
				
				this.set;
				this.turn;
				this.step;
				
				
			}
	}
	
	//using Patterns to control movement
	default { 
			function = {
					event = stream.next(event);
						
						this.set;
						this.turn;
						this.step;
			}
	}
	

	//create dots with patterns 
	points { arg pattern;
			
			function = {
				event = stream.next(event);
				this.set;
			}
	
	}
	//using Patterns to control movement, moving back and fourth
	worm { 
			var rout;
			
			
			
			rout  = Routine({
						var  length;
						
						
						
						
						
						loop({
							event = stream.next(event);
							length = event.at(\length);
							this.push;
							yield(0);
									//#{ { Line.kr(0.1, 0, 0.3)*SinOsc.ar(300+200.rand2) }.play; }.send;
						
							(length-1).do({ arg i;
								event = stream.next(event);
								this.push;
								yield(0);
							});
							length.do({ arg i;
								this.pop;
								if(event.notNil, {
									event.use({
										~direction = ~direction + 1.0;//turn 180 degrees
										~value = ~value2;
									});
								});
								yield(0);
								
							});
						});
					});
					
				
			function = { rout.next; this.set; this.step; };
				
				
	}
	
	
	
	wipe { 
		
		function = Routine.new({ 
		
			loop({
				array2D.cols.do({ arg i;
					array2D.collectCol(i, { arg item; 
									item * (event.at(\mul) ? 0.8);
									});
									yield(0);
					})
				});
	})
	
	}
	
			
	
}


GUIAnt : Ant {
	var <>field;
	*new { arg field, mode=\stream,stream;
			^super.new(field.array2D, mode, stream).field_(field)
	}
	
	
	next { arg inval; 
			function.value(inval);
			event.use({
				field.at(~y, ~x).colorBase_(~color);
			});
			
	}

	
}
