//JulianRohrhuber 2001/05 version 0.3
//a number of classes that create GUI similar to NumericalView
//keeping the control key down shows the number.

Dimmer : ControlView {
	var <>value, <>lastVal; 
	var <>scale, <>maxColor, <>minColor;
	var <>numericalMode=false;
	
	*new { arg pen, bounds, control, margin, layoutSize, value=0.0, scale=90;
		^super.new(pen, bounds, control, margin, layoutSize).value_(value).scale_(scale).init
	}
	init {
		this.bounds = bounds;
		maxColor = pen.foreColor ?? Color.blue;
		minColor = pen.backColor ?? Color.black;
		this.setColor;
	}
	
	setColor { 
		var r, g, b, r0, g0, b0;
		value = control.get;
		r = maxColor.red; g = maxColor.green; b = maxColor.blue;
		r0 = minColor.red; g0 = minColor.green; b0 = minColor.blue;
		pen.foreColor_(Color.new((r-r0)*value + r0, (g-g0)*value + g0, (b-b0)*value + b0));
	}
	
	draw {
		this.setColor;
		pen.use({ Pen.action = \fill; bounds.draw });
		if(numericalMode, {this.drawValue});
			
	}
	drawValue {
		var r, g, b;
		r = pen.foreColor.red;	g = pen.foreColor.green; 	b = pen.foreColor.blue;
		pen.use({ 	Pen.foreColor = Color.new(255-r, 255-g, 255-b);
				Pen.action = \fill; 
				Pen.textSize = 9;
				control.getMapped.trunc(0.01).asString.drawLeftJustIn(bounds); 
		});
	}
	
	mouseDown { arg where, event;
		if (bounds.containsPoint(where), { 
			
			this.storeValues;
			this.respond(where, event);
			^this; 
		},{ 
			^nil 
		});
	}
	
	mouseMoved { arg where, event;
		this.respond(where, event);
	}
	
	mouseUp { arg where, event;
		
		this.storeValues;
		numericalMode = false;
	}
	
	calcValue { arg where;
			value = (lastVal + (bounds.center.y - where.y / scale)).clip(0, 1.0);
	}
	
	respond { arg where, event; 
		
		if(event.controlKey, { numericalMode = true }, { numericalMode = false } );
		this.calcValue(where);
		control.set(value);
		
	}
	
	storeValues {  lastVal = value }	
	
	layout { arg argBounds;
		this.bounds = argBounds.insetByRect(margin);
	}
}




FieldDimmer : Dimmer { 
	var <>field;
	
	*new { arg pen, bounds, control, margin, layoutSize, value=0.0, scale=90, field;
		^super.new(pen, bounds, control, margin, layoutSize, value, scale).field_(field)
	}
	
	
	storeValues {  field.lastVal = value;
				lastVal = value;
	 }
	
	mouseUp { arg where, event;
		this.storeValues;
		numericalMode = false;
	}
	
	mouseDown { arg where, event;
		var val; 
		if (bounds.containsPoint(where), { 
			
			val = if(event.cmdKey, { field.lastVal }, { value });
			value = val;
			lastVal = val;
			this.respond(where, event);
			
			^this; 
		},{ 
			^nil 
		});
		
	}
	
	
}


FieldDimmer2 : FieldDimmer {
	var <>index=0, <>lastIndex=0;
	//color information on horizontal scale
	//the color information is stored in the Field2 it belongs to...
	
	*new { arg pen, bounds, control, margin, layoutSize, value=0.0, scale=90, field;

		^super.new(pen, bounds, control, margin, layoutSize, value=0.0, scale=90, field);
	}

	
	setColor { 
		var r, g, b;
		maxColor = field.colors.at(index.asInteger);
		r = maxColor.red; g = maxColor.green; b = maxColor.blue;
		pen.foreColor_(Color.new(r*value, g*value, b*value));
	}
	
	calcValue { arg where;
		var size;
		size = field.colors.size;
		value = (lastVal + (bounds.center.y - where.y / scale)).clip(0, 1.0);
		index = (lastIndex + (bounds.center.x - where.x / scale * size )).asInteger.wrap(0, size-1);
	}
		
	storeValues {  	field.lastVal = value;
				lastVal = value;
				lastIndex = index;
	}
	
	
}
		
