//some extended functionality for Array2D
//Julian Rohrhuber 2001/05

+Array2D {
	
	
	wrapAt { arg row, col;  
		row = row.wrap(0, rows-1);
		col = col.wrap(0, cols-1);
		^this.at(row,col);
	}
	wrapPut { arg row, col, val; 
		row = row.wrap(0, rows-1);
		col = col.wrap(0, cols-1);
		this.put(row, col, val);
	}
	clipPut { arg row, col, val; 
		row = row.clip(0, rows-1);
		col = col.clip(0, cols-1);
		this.put(row, col, val);
	}
	row { arg row; ^Array.fill(cols, { arg col; this.wrapAt(row, col) }) }
	col { arg col; ^Array.fill(rows, { arg row; this.wrapAt(row, col) }) }
	do { arg function;
		rows.do({ arg row;
			cols.do({ arg col;
				function.value(this.at(row, col), row, col)
			})
		})
	}
			
	
	*fill { arg rows, cols, func;
		var array2D;
		array2D = this.new(rows, cols);
		rows.do({ arg row;
			cols.do({ arg col;
				array2D.put(row, col, func.value(row, col))
			})
		});
		^array2D
		
	}
		
	collect { arg func;
		
		rows.do({ arg row;
			cols.do({ arg col;
				this.put(row, col, func.value(this.at(row, col), row, col))
			})
		});
		
	}
	
	putCol { arg col, array, offset=0;
		
		array.do({ arg item, i; 
			this.wrapPut(i + offset, col, item);
		});
		
	}
	collectCol { arg col, func;
		
		rows.do({ arg i;
			var item;
			item = this.at(i, col);
			this.put(i, col, func.value(item, i));
		});
		
	}
	doCol { arg col, func;
		rows.do({ arg i;
			var item;
			item = this.at(i, col);
			func.value(item, i);
			})
	}
	
	putRow { arg row, array, offset=0, selection;
		
		
		if(selection.isNil, {
			array.do({ arg item, i; 
				this.wrapPut(row, i+offset, item);
			}) 
		}, {
			array.do({ arg item, i; 
				if(selection.at(i) > 0, {
				this.wrapPut(row, i+offset, item);
				});
			});
		});
		
	}
	
	collectRow { arg row, func, selection;
		var item;
		
		if(selection.isNil, {
			cols.do({ arg i;
				
				item = this.at(row, i);
				this.put(row, i, func.value(item, i));
		})
		}, {
			cols.do({ arg i;
				
				if(selection.at(i) > 0, {
					item = this.at(row, i);
					this.put(row, i, func.value(item, i));
				});
			});
		});
		
	}
	
	doRow { arg row, func, selection;
		var item;
		if(selection.isNil, {
				cols.do({ arg i;
					
					item = this.at(row, i);
					func.value(item, i);
				})
			}, {
				cols.do({ arg i;
					if(selection.at(i) > 0, {
						
						item = this.at(row, i);
						func.value(item, i);
					})
				})
			});
			
		
	}
	
	rotateCol { arg col, steps=1, length, offset=0;
		var array;
		length = length ? this.rows;
		array = Array.fill(length, { arg i; this.wrapAt(i + offset, col) });
		array = array.rotate(steps);
		this.putCol(col, array, offset);
	}
}


