
//a basic implementation of the language 'Turtle' (LOGO).
//for SC3d
//a turtle is represented by a TurtleEvent that is being 
//modified by a string of commands that can be put into a
//ListPattern.
//the characters of relevance are right now:
//F  	//move one step forward drawing a line
//f	//move one step forward without drawing a line
//+	//turn one step right
//-	//turn one step left
//[]	//execute this first and then return

//by Julian Rohrhuber with support by Michael Renton, may 01.
//more information on this topic: http://swiki.hfbk.uni-hamburg.de:8080/MusicTechnology/Turtle


Turtle : Stream {
	
	//very similar to EventStream, only that the event is modified further and further
	//so it never goes home unless you push it in the beginning so that later on you can pop
	//it back.
	
	var <>stream, <>event, <>stack;
	*new { arg stream, event; ^super.new.stream_(stream).event_(event).initStack }
	
	initStack { stack = LinkedList.new; }
	
	push {
		stack.add(event.copy);
	}
	
	pop {
		event =  stack.pop ? event;
	}
	
}



L_Turtle : Turtle { //a basic turtle language interface
	var <>cmd, <>dict, <>lines, <>argCollect=false;
	*new { arg stream, event;
			^super.new( stream, event).init 
	}
	
	init { 
		lines = Lines.new(80);
		event = event ?? TurtleEvent.protoEvent;
		dict = Dictionary[
				\F -> { this.moveDraw },
				\f -> { this.move },
				'+' -> { this.turnRight },
				'-' -> { this.turnLeft },
				'[' -> { this.push },
				']' -> { this.pop }
				//'{' -> { argCollect = true },
				//'}' -> { argCollect = false; cmd.interpret },
				]
	}
	
	next {
		var nextEvent;
		
		
		nextEvent = stream.next(event);//make sure event is never nil
		
		if(nextEvent.notNil, {
			event = nextEvent;
			event.finishEvent;
			//future arg input parser...
			if(argCollect, {
				cmd = cmd.asString ++ event.at(\cmd).asString
				}, {
				cmd = event.at(\cmd);
		});
		dict.at(cmd.asSymbol).value;
		});
		
	}
	move {
		event.use({ ~position = ~position + Point(~incstep * ~angle.cos, ~incstep * ~angle.sin) })
	}
	moveDraw {
		lines.add(event.at(\position));
		this.move;
		lines.add(event.at(\position));
	}
	
	turnRight {
		event.use({ ~angle = ~angle + ~incangle })
	}
	turnLeft {
		event.use({ ~angle = ~angle - ~incangle })
	}

	draw { event.at(\pen).use({ lines.draw }) 
	
	}
		
}

L_Modifier { //can be used in Plsys to allow rewriting
	var <>name, <>function;
	*new { arg name, func;
		^super.newCopyArgs(name, func)
	}
	asSymbol { ^name } //support for Plsys identity
	value { arg ... args; ^function.performList(\value, args) }

}

//todo:
// lines to be an array (pen can change) or should lines be in the event?
// 3D?
// arg passing